import numpy as np
from typing import List
from astropy.io import fits
import matplotlib.pyplot as plt

from blue_objects import file, path
from blue_objects.graphics.gif import generate_animated_gif

from hubblescope import NAME
from hubblescope.logger import logger


def load_fit_file(
    filename: str,
    plot: bool = False,
    cmap: str = "hot",
) -> List[np.ndarray]:
    list_of_images = []
    with fits.open(filename) as hdul:
        hdul.info()
        logger.info(f"{NAME}.load_fit_file({filename}): {len(hdul)} item(s)")

        images = []
        first_to_show = True
        for index, item in enumerate(hdul):
            if item.data is None:
                logger.info(f"#{index}: Null")
                continue

            image = item.data
            logger.info(f"#{index}: {image.shape}")

            if len(image.shape) != 2:
                logger.info("skipped.")
                continue

            images += [image]

            margin = 5

            image_flattenned = image.flatten()
            image_flattenned = image_flattenned[~np.isnan(image_flattenned)]
            if not image_flattenned.shape:
                logger.warning("empty (likely all nan) image, skipped.")
                continue
            vmin = np.percentile(image_flattenned, margin)
            vmax = np.percentile(image_flattenned, 100 - margin)

            logger.info(f"#{index}: {vmin:.3f} - {vmax:.3f}")

            histogram, bin_edges = np.histogram(
                image_flattenned,
                bins=256,
                range=(vmin, vmax),
            )

            bins = (bin_edges[:-1] + bin_edges[1:]) / 2

            plt.figure(figsize=(10, 5))

            plt.subplot(121)
            plt.imshow(
                image,
                cmap=cmap,
                vmin=vmin,
                vmax=vmax,
            )
            plt.colorbar(cmap=cmap)
            plt.title(f"{file.name_and_extension(filename)}/{index}")
            plt.xlabel(path.name(file.path(filename)))

            plt.subplot(122)
            plt.plot(bins, histogram, color="b")
            plt.xlabel("value")
            plt.ylabel("frequency")
            plt.grid(True)
            image_filename = file.add_suffix(
                file.add_extension(filename, "png"),
                f"{index}",
            )
            plt.savefig(image_filename)
            list_of_images += [image_filename]
            if plot and first_to_show:
                first_to_show = False
                plt.show()
            else:
                plt.close()

        generate_animated_gif(
            list_of_images,
            file.add_extension(filename, "gif"),
            frame_duration=500,
        )

    return images



NAME = "hubblescope"

ICON = "🔭"

DESCRIPTION = f"{ICON} tools to access and process Hubble Space Telescope imagery and other datasets on AWS Open Data Registry."

VERSION = "4.42.1"

MARQUEE = "https://github.com/kamangir/hubble/raw/main/assets/hst/u4ge0106r_c0m.gif"



from typing import Any
import os

from blue_options import string
from blue_objects import file
from blue_objects.env import abcli_path_git


# https://registry.opendata.aws/hst/
# https://github.com/awslabs/open-data-registry/blob/main/datasets/hst.yaml
def get(
    dataset_name: str,
    what: str,
    object_name: str = "",
) -> Any:
    if what == "auth":
        resources = get(dataset_name, "resource:S3 Bucket")
        if not resources:
            return "auth:resource-not-found"

        RequesterPays = resources[0].get("RequesterPays", False)

        return "--requester-pays" if RequesterPays else "--no-sign-request"

    if what == "metadata":
        return file.load_yaml(
            get(
                dataset_name,
                "metadata_filename",
            )
        )[1]

    if what == "metadata_filename":
        return os.path.join(
            abcli_path_git,
            f"open-data-registry/datasets/{dataset_name}.yaml",
        )

    if what.startswith("resource:"):
        resource_type = string.after(what, "resource:")

        metadata = get(dataset_name, "metadata")
        if not metadata:
            return [{"resource:metadata-not-found": True}]

        return [
            resource
            for resource in metadata.get("Resources", {})
            if resource.get("Type") == resource_type
        ]

    if what == "s3_uri":
        resources = get(dataset_name, "resource:S3 Bucket")
        if not resources:
            return "s3_uri:resource-not-found"

        ARN = resources[0].get("ARN", "")

        # arn:aws:s3:::stpubdata/hst
        return "s3://{}/{}".format(
            string.after(ARN, "arn:aws:s3:::"),
            f"{object_name}/" if object_name else "",
        )

    return f"{what}-not-found"



from tqdm import tqdm

from blueness import module
from blue_objects import file, objects

from hubblescope import NAME
from hubblescope.fits import load_fit_file
from hubblescope.logger import logger


NAME = module.name(__file__, NAME)


def ingest(
    object_name: str,
    dataset_name: str,
    hubble_object_name: str,
    plot: bool = False,
) -> bool:
    logger.info(
        "{}.ingest({} :: {} [{}])".format(
            NAME,
            dataset_name,
            object_name,
            hubble_object_name,
        )
    )

    if dataset_name in ["hst"]:
        logger.info("ingesting *fit")
        for filename in tqdm(objects.list_of_files(object_name)):
            if file.extension(filename) == "fits":
                load_fit_file(filename, plot=plot)

    return True
